#version 330

layout(std140) uniform Matrices{

	mat4 projMatrix;
	mat4 viewMatrix;
	mat4 modelMatrix;
};

layout(std140) uniform Material{
	vec4 materialDiffuse;
	vec4 materialAmbient;
	vec4 materialSpecular;
	vec4 materialEmissive;
	float materialShininess;
	int texCount;
};

layout(std140) uniform PreviousMatrices{

	mat4 prevProjMatrix;
	mat4 prevViewMatrix;
	mat4 prevModelMatrix;
};

layout(std140) uniform PointLight{
	vec3 lightPosition;
	vec4 lightColor;
	vec3 lightAmbientCoefficient;
	vec3 lightAttenuation;
	int uniformBlockIndex;
};

uniform sampler2D tex;
uniform sampler2D depthBuffer;
uniform sampler2D colorBuffer;

uniform vec3 cameraPosition;
uniform vec3 cameraDirection;

in vec3 fragVert;
in vec2 fragTexCoord;
in vec3 fragNormal;

out vec4 fragColor;

void main()
{
	vec3 p = lightPosition;
	vec3 normal = normalize(transpose(inverse(mat3(modelMatrix))) * fragNormal);
	vec3 surfacePos = vec3(modelMatrix * vec4(fragVert, 1));

	vec4 surfaceColor = materialDiffuse;
	if (texCount != 0)
	{
		surfaceColor = texture(tex, fragTexCoord);
	}

	vec3 surfaceToLight = normalize(p - surfacePos);
	vec3 surfaceToCamera = normalize(cameraPosition - surfacePos);

	//ambient
	vec3 ambient = lightAmbientCoefficient.x * surfaceColor.rgb * lightColor.rgb;

	//diffuse
	float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight));
	vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * lightColor.rgb;

	//specular
	float specularCoefficient = 0.0;
	if (diffuseCoefficient > 0.0)
		specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), materialShininess);
	vec3 specular = specularCoefficient * materialSpecular.rgb * lightColor.rgb;

	//attenuation
	float distanceToLight = length(p - surfacePos);
	float attenuation = 1.0 / (1.0 + lightAttenuation.x * pow(distanceToLight, 2));

	//linear color (color before gamma correction)
	vec3 linearColor = ambient + attenuation*(diffuse + specular);

	//final color (after gamma correction)
	vec3 gamma = vec3(1.0 / 2.2);
	fragColor = vec4(pow(linearColor, gamma), surfaceColor.a);
	//fragColor = vec4(1.0);

}