#version 330

 layout (std140) uniform Matrices {
 
    mat4 projMatrix;
    mat4 viewMatrix;
    mat4 modelMatrix;
};

layout (std140) uniform Material {
    vec4 materialDiffuse;
    vec4 materialAmbient;
    vec4 materialSpecular;
    vec4 materialEmissive;
    float materialShininess;
    int texCount;
};
 
  layout (std140) uniform PreviousMatrices {
 
    mat4 prevProjMatrix;
    mat4 prevViewMatrix;
    mat4 prevModelMatrix;
};

 layout (std140) uniform PointLight {
   vec3 lightPosition;
   vec4 lightColor;
   vec3 lightAmbientCoefficient; 
   vec3 lightAttenuation;
   int uniformBlockIndex;
 };

uniform sampler2D tex;
uniform sampler2D depthBuffer;
uniform sampler2D colorBuffer;

uniform vec3 cameraPosition;
uniform vec3 cameraDirection;
uniform float fps;

//in vec3 fragVert;
//in vec2 fragTexCoord;
//in vec3 fragNormal;

noperspective in vec2 fragTex;

out vec4 fragColor;
 
void main()
{


   // Get the depth buffer value at this pixel.  
   float zOverW = texture(depthBuffer, fragTex).r;  
	// H is the viewport position at this pixel in the range -1 to 1.  
   vec4 H = vec4(fragTex.x * 2 - 1, (1 - fragTex.y) * 2 - 1,  
	zOverW, 1);  
	// Transform by the view-projection inverse.  
   vec4 D = (H * inverse(projMatrix * viewMatrix));  
	// Divide by w to get the world position.  
   vec4 worldPos = D / D.w;  

    // Current viewport position  
   vec4 currentPos = H;  
   // Use the world position, and transform by the previous view-  
   // projection matrix.  
   vec4 previousPos = worldPos * (prevProjMatrix * prevViewMatrix);  
   // Convert to nonhomogeneous points [-1,1] by dividing by w.  
   previousPos /= previousPos.w;  
   // Use this frame's position and last frame's to compute the pixel  
   // velocity.  

   float mblurScale = fps / 60;

   vec2 velocity = mblurScale * ((currentPos.xy - previousPos.xy)/8.f);  
	

	int nSamples = 16; 
	
	int div = 1;
	for(int i = 1; i < nSamples; ++i)
	{
		vec2 offset = velocity * (float(i) / float(nSamples - 1) - 0.5);

		if( (fragTex+offset).x > 0.0 && (fragTex+offset).x < 1.0 &&
		(fragTex+offset).y > 0.0 && (fragTex+offset).y < 1.0)
		{
			//fragColor = vec4(1.0, 1.0, 0.0, 0.0);
			fragColor += texture(colorBuffer, fragTex + offset);
			div++;
		}
		else
		{
			fragColor += texture(colorBuffer, fragTex);
			//fragColor = vec4(1.0, 0.0, 0.0, 0.0);
		}

	}

	fragColor /= float(nSamples);
	//fragColor = vec4(1.0, 1.0, 0.0, 0.0);


	
}