#version 420
 
 layout (std140) uniform Matrices {
 
    mat4 projMatrix;
    mat4 viewMatrix;
    mat4 modelMatrix;
};

layout (std140) uniform Material {
    vec4 materialDiffuse;
    vec4 materialAmbient;
    vec4 materialSpecular;
    vec4 materialEmissive;
    float materialShininess;
    int texCount;
};
 
  layout (std140) uniform PreviousMatrices {
 
    mat4 prevProjMatrix;
    mat4 prevViewMatrix;
    mat4 prevModelMatrix;
};

 layout (std140) uniform PointLight {
   vec3 lightPosition;
   vec4 lightColor;
   vec3 lightAmbientCoefficient; 
   vec3 lightAttenuation;
   int uniformBlockIndex;
 };

uniform sampler2D tex;
uniform sampler2D nmTex;

uniform vec3 cameraPosition;
uniform vec3 cameraDirection;

in vec3 fragVert;
in vec2 fragTexCoord;
in vec3 fragNormal;
in vec3 LightDir;
in vec3 ViewDir;

out vec4 fragColor;
 

vec3 phongModel(vec3 norm, vec3 diffR)
{
	vec3 r = reflect(-LightDir, norm);
	vec3 ambient = lightColor.rgb * materialAmbient.rgb;
	float sDotN = max(dot(LightDir, norm), 0.0);
	vec3 diffuse = lightColor.rgb * diffR * sDotN;

	vec3 spec = vec3(0.0);
	if(sDotN > 0.0)
	{
		spec = lightColor.rgb * materialSpecular.rgb * pow(max(dot(r, ViewDir), 0.0), materialShininess);
	}

	return ambient + diffuse + spec;
}

void main()
{

	vec4 normalMap = ((2.0 * texture(nmTex, fragTexCoord)) - vec4(1.0));
	//vec4 texColor = materialDiffuse;
	//if(texCount != 0)
	//{
	//	texColor = texture(nmTex, fragTexCoord);
	//}

	//fragColor = vec4(phongModel(normal.xyz, texColor.rgb), 1.0);
	//fragColor = texColor;



	vec3 p = lightPosition;
	vec3 normal = normalize(transpose(inverse(mat3(modelMatrix))) * fragNormal);
	vec3 standardNormal = normal;
    vec3 surfacePos = vec3(modelMatrix * vec4(fragVert, 1));

	vec3 surfaceToLight = normalize(p - surfacePos);
    vec3 surfaceToCamera = normalize(cameraPosition - surfacePos);

	vec4 surfaceColor = materialDiffuse;
	if(texCount != 0)
	{
		surfaceColor = texture(tex, fragTexCoord);
		normal = normalMap.xyz;

		surfaceToLight = normalize(LightDir);
		surfaceToCamera = normalize(ViewDir);
		//fragColor = vec4(phongModel(normal.xyz, surfaceColor.rgb), 1.0);
	}
	//else
	//{
	//ambient
    vec3 ambient = lightAmbientCoefficient.x * surfaceColor.rgb * lightColor.rgb;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * lightColor.rgb;

    //specular
    float specularCoefficient = 0.0;
    if(diffuseCoefficient > 0.0)
        specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), materialShininess);
    vec3 specular = specularCoefficient * materialSpecular.rgb * lightColor.rgb;

    //attenuation
    float distanceToLight = length(p - surfacePos);
    float attenuation = 1.0 / (1.0 + lightAttenuation.x * pow(distanceToLight, 2));
        
    //linear color (color before gamma correction)
    vec3 linearColor = ambient + attenuation*(diffuse + specular);
    
    //final color (after gamma correction)
    vec3 gamma = vec3(1.0/2.2);
    fragColor = vec4(pow(linearColor, gamma), surfaceColor.a);
	//fragColor = vec4(ViewDir, 1.0);
	//}
}