#version 420
 
layout (std140) uniform Matrices {
 
    mat4 projMatrix;
    mat4 viewMatrix;
    mat4 modelMatrix;
};

 layout (std140) uniform PreviousMatrices {
 
    mat4 prevProjMatrix;
    mat4 prevViewMatrix;
    mat4 prevModelMatrix;
};
 layout (std140) uniform PointLight {
   vec3 lightPosition;
   vec4 lightColor;
   vec3 lightAmbientCoefficient; 
   vec3 lightAttenuation;
   int uniformBlockIndex;
 };

uniform sampler2D tex;
uniform sampler2D nmTex;

uniform vec3 cameraPosition;
uniform vec3 cameraDirection;

in vec3 position;
in vec3 normal;
in vec2 uvPosition;
in vec3 tangent;
in vec3 bitangent;

out vec3 fragVert;
out vec2 fragTexCoord;
out vec3 fragNormal;
out vec3 fragTangent;
out vec3 LightDir;
out vec3 ViewDir;

void main()
{
	mat3 normalMatrix = (transpose(inverse(mat3(viewMatrix * modelMatrix))));

	vec3 norm = normalize(normalMatrix * normal);
	vec3 tang = normalize(normalMatrix * tangent);
	vec3 binormal = normalize(normalMatrix * bitangent);

	mat3 toObjectLocal = mat3(
		tang.x, binormal.x, norm.x,
		tang.y, binormal.y, norm.y,
		tang.z, binormal.z, norm.z);

	//mat3 toObjectLocal = mat3(
	//	tang.x, tang.y, tang.z,
	//	binormal.x, binormal.y, binormal.z,
	//	norm.x, norm.y, norm.z);

	vec3 pos = vec3((viewMatrix * modelMatrix) * vec4(position, 1.0));
	vec3 lightPositionEyeCoords = vec3((viewMatrix) * vec4(lightPosition, 1.0));
	vec3 cameraPositionEyeCoords = vec3((viewMatrix * modelMatrix) * vec4(cameraPosition, 1.0));

	LightDir = (toObjectLocal * (lightPositionEyeCoords - pos));
	ViewDir = toObjectLocal * (-pos);

    fragTexCoord = uvPosition;
    fragNormal = normal;
    fragVert = position;
	fragTangent = tangent;
    gl_Position =  projMatrix  * viewMatrix * modelMatrix * vec4(position, 1);

}